@@ -166,10 +166,15 @@ struct NDS_fw_config_data {
 
 extern NDSSystem nds;
 
+#ifdef GDB_STUB
 int NDS_Init( struct armcpu_memory_iface *arm9_mem_if,
               struct armcpu_ctrl_iface **arm9_ctrl_iface,
               struct armcpu_memory_iface *arm7_mem_if,
               struct armcpu_ctrl_iface **arm7_ctrl_iface);
+#else
+int NDS_Init ( void);
+#endif
+
 void NDS_DeInit(void);
 void
 NDS_FillDefaultFirmwareConfigData( struct NDS_fw_config_data *fw_config);
