@@ -28,6 +28,7 @@ extern "C" {
 
 #define FS_IS_DIR 1
 
+#define FS_ERR_UNKNOWN -1
 #define FS_ERR_NO_MORE_FILES 1
 
 extern const char FS_SEPARATOR;
@@ -36,6 +37,7 @@ typedef struct {
 	char cFileName[256];
 	char cAlternateFileName[14];
 	u32 flags;
+        u32 fileSize;
 } FsEntry;
 
 void * FsReadFirst(const char * path, FsEntry * entry);
