@@ -19,14 +19,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
+
 #ifndef DESMUME_COCOA
 
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <gl\gl.h>
-#include <gl\glext.h>
-#include <math.h>
-#include <stdlib.h>
 #include "..\debug.h"
 #include "..\MMU.h"
 #include "..\bits.h"
@@ -34,7 +35,6 @@
 
 #else
 
-
 #include <OpenGL/gl.h>
 #include <OpenGL/glext.h>
 #include "../debug.h"
@@ -50,7 +50,8 @@
 #define fix2float(v)    (((float)((s32)(v))) / (float)(1<<12))
 #define fix10_2float(v) (((float)((s32)(v))) / (float)(1<<9))
 
-static unsigned char  GPU_screen3D[256*256*4]={0};
+static unsigned char  GPU_screen3D		[256*256*3]={0};
+static unsigned char  GPU_screen3DMask	[256*256]={0};
 
 // Acceleration tables
 static float*		float16table = NULL;
@@ -101,6 +102,7 @@ static float invTexWidth  = 1.f;
 static float invTexHeight = 1.f;
 static float fogColor[4] = {0.f};
 static float fogOffset = 0.f;
+static float alphaTestRef = 0.01f;
 
 static unsigned long clCmd = 0;
 static unsigned long clInd = 0;
@@ -114,6 +116,14 @@ static unsigned int vtxFormat;
 static unsigned int textureFormat=0, texturePalette=0;
 static unsigned int lastTextureFormat=0, lastTexturePalette=0;
 
+typedef struct
+{
+	unsigned int color;		// Color in hardware format
+	unsigned int direction;	// Direction in hardware format
+} LightInformation;
+
+LightInformation g_lightInfo[4] = { 0 };
+
 #ifndef DESMUME_COCOA
 extern HWND		hwnd;
 #endif
@@ -121,6 +131,7 @@ extern HWND		hwnd;
 char NDS_glInit(void)
 {
 	int i;
+
 #ifndef DESMUME_COCOA
 	HDC						oglDC = NULL;
 	HGLRC					hRC = NULL;
@@ -156,11 +167,15 @@ char NDS_glInit(void)
 
 	glEnable(GL_DEPTH_TEST);
 	glEnable(GL_TEXTURE_2D);
+	glEnable(GL_ALPHA_TEST);
+	glEnable(GL_STENCIL_TEST);
 
-	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
-	glColor3f (1.f, 1.f, 1.f);
+	glAlphaFunc		(GL_GREATER, 0.01f);
+	glStencilOp		(GL_KEEP, GL_KEEP, GL_INCR);
+	glClearColor	(0.0f, 0.0f, 0.0f, 0.0f);
+	glColor3f		(1.f, 1.f, 1.f);
 
-	glGenTextures (1, &oglTextureID);
+	glGenTextures (1, (GLuint*)&oglTextureID);
 
 	glViewport(0, 0, 256, 192);
 
@@ -208,15 +223,27 @@ char NDS_glInit(void)
 
 void NDS_glViewPort(unsigned long v)
 {
-	glViewport( (v&0xFF), ((v>>8)&0xFF), ((v>>16)&0xFF), (v>>24));
+	if(beginCalled)
+		glEnd();
+
+	glViewport( (v&0xFF), ((v>>8)&0xFF), ((v>>16)&0xFF)+1, (v>>24)+1);
+
+	if(beginCalled)
+		glBegin(vtxFormat);
 }
 
 void NDS_glClearColor(unsigned long v)
 {
+	if(beginCalled)
+		glEnd();
+
 	glClearColor(	((float)(v&0x1F))/31.0f,
 					((float)((v>>5)&0x1F))/31.0f,
 					((float)((v>>10)&0x1F))/31.0f,
 					((float)((v>>16)&0x1F))/31.0f);
+
+	if(beginCalled)
+		glBegin(vtxFormat);
 }
 
 void NDS_glFogColor(unsigned long v)
@@ -234,17 +261,29 @@ void NDS_glFogOffset (unsigned long v)
 
 void NDS_glClDepth()
 {
+	if(beginCalled)
+		glEnd();
+
 	glClear(GL_DEPTH_BUFFER_BIT);
+
+	if(beginCalled)
+		glBegin(vtxFormat);
 }
 
 void NDS_glClearDepth(unsigned long v)
 {
 	u32 depth24b;
 
+	if(beginCalled)
+		glEnd();
+
 	v		&= 0x7FFFF;
 	depth24b = (v*0x200)+((v+1)/0x8000);
 
 	glClearDepth(depth24b / ((float)(1<<24)));
+
+	if(beginCalled)
+		glBegin(vtxFormat);
 }
 
 void NDS_glMatrixMode(unsigned long v)
@@ -266,17 +305,21 @@ void SetMatrix (void)
 {
 	if (mode < 2)
 	{
-		glEnd();
+		if (beginCalled)
+			glEnd();
 		glMatrixMode (matrixMode[mode]);
 		glLoadMatrixf(mtxCurrent[mode]);
 		glBegin (vtxFormat);
+		beginCalled = 1;
 	}
 	else if (mode == 2)
 	{
-		glEnd();
+		if (beginCalled)
+			glEnd();
 		glMatrixMode (matrixMode[1]);
 		glLoadMatrixf(mtxCurrent[1]);
 		glBegin (vtxFormat);
+		beginCalled = 1;
 	}
 }
 
@@ -443,13 +486,14 @@ static __inline void SetupTexture (unsigned int format, unsigned int palette)
 		unsigned int sizeY = (1<<(((format>>23)&0x7)+3));
 		unsigned int mode = (unsigned short)((format>>26)&0x7);
 		unsigned char * adr = (unsigned char *)(ARM9Mem.ARM9_LCD + ((format&0xFFFF)<<3));
-		unsigned short param = (unsigned short)((format>>30)&0xF);
-		unsigned short param2 = (unsigned short)((format>>16)&0xF);
+		//unsigned short param = (unsigned short)((format>>30)&0xF);
+		//unsigned short param2 = (unsigned short)((format>>16)&0xF);
 		unsigned int imageSize = sizeX*sizeY;
 		unsigned int paletteSize = 0;
 		unsigned int palZeroTransparent = (1-((format>>29)&1))*255; // shash: CONVERT THIS TO A TABLE :)
 		unsigned int x=0, y=0;
-		unsigned char * dst = texMAP, *src = NULL;
+		unsigned char * dst = texMAP;
+		//unsigned char *src = NULL;
 
 		if (mode == 0)
 			glDisable (GL_TEXTURE_2D);
@@ -799,7 +843,7 @@ static __inline void SetupTexture (unsigned int format, unsigned int palette)
 
 			case 2:
 			{
-				u32 *src = (u32*)texMAP, *dst = (u32*)texMAP2;
+				u32 *src = (u32*)texMAP;//, *dst = (u32*)texMAP2;
 
 				for (y = 0; y < sizeY; y++)
 				{
@@ -915,7 +959,7 @@ void NDS_glBegin(unsigned long v)
 		*/
 
 		colorRGB[3] = colorAlpha;
-		glColor4iv (colorRGB);
+		glColor4iv ((GLint*)colorRGB);
 	}
 	else
 	{
@@ -961,7 +1005,7 @@ void NDS_glColor3b(unsigned long v)
 	colorRGB[0] =  (v&0x1F) << 26;
 	colorRGB[1] = ((v>>5)&0x1F) << 26;
 	colorRGB[2] = ((v>>10)&0x1F) << 26;
-	glColor4iv (colorRGB);
+	glColor4iv ((GLint*)colorRGB);
 }
 
 static __inline void  SetVertex()
@@ -970,14 +1014,14 @@ static __inline void  SetVertex()
 
 	if (texCoordinateTransform == 3)
 	{
-		int s2 =	(int)((	coord[0]*mtxCurrent[3][0] +
-							coord[1]*mtxCurrent[3][4] +
-							coord[2]*mtxCurrent[3][8]) + s);
-		int t2 =	(int)((	coord[0]*mtxCurrent[3][1] +
-							coord[1]*mtxCurrent[3][5] +
-							coord[2]*mtxCurrent[3][9]) + t);
+		float s2 =((coord[0]*mtxCurrent[3][0] +
+					coord[1]*mtxCurrent[3][4] +
+					coord[2]*mtxCurrent[3][8]) + s);
+		float t2 =((coord[0]*mtxCurrent[3][1] +
+					coord[1]*mtxCurrent[3][5] +
+					coord[2]*mtxCurrent[3][9]) + t);
 
-		glTexCoord2i (s2, t2);
+		glTexCoord2f (s2, t2);
 	}
 
 	MatrixMultVec4x4 (mtxCurrent[1], coordTransformed);
@@ -1050,20 +1094,16 @@ int NDS_glGetNumVertex (void)
 void NDS_glGetLine (int line, unsigned short * dst)
 {
 	int i;
-	u8 *screen3D = (u8 *)&GPU_screen3D[(192-(line%192))*256*4];
+	u8	*screen3D		= (u8 *)&GPU_screen3D		[(192-(line%192))*256*3],
+		*screen3DMask	= (u8 *)&GPU_screen3DMask	[(192-(line%192))*256];
 
 	for(i = 0; i < 256; i++)
 	{
-		u32	r = screen3D[i*4+0],
-			g = screen3D[i*4+1],
-			b = screen3D[i*4+2],
-			a = screen3D[i*4+3];
+		u32	r = screen3D[i*3+0],
+			g = screen3D[i*3+1],
+			b = screen3D[i*3+2];
 
-		r = (r*a)/255;
-		g = (g*a)/255;
-		b = (b*a)/255;
-
-		if (r != 0 || g != 0 || b != 0)
+		if (screen3DMask[i] > 0)
 		{
 			dst[i] = (((r>>3)<<10) | ((g>>3)<<5) | (b>>3));
 		}
@@ -1082,7 +1122,8 @@ void NDS_glFlush(unsigned long v)
 	clInd = 0;
 
 	glFlush();
-	glReadPixels(0,0,256,192,GL_BGRA,GL_UNSIGNED_BYTE,GPU_screen3D);
+	glReadPixels(0,0,256,192,GL_BGR_EXT,		GL_UNSIGNED_BYTE,GPU_screen3D);
+	glReadPixels(0,0,256,192,GL_STENCIL_INDEX,	GL_UNSIGNED_BYTE,GPU_screen3DMask);
 
 	numVertex = 0;
 
@@ -1091,12 +1132,12 @@ void NDS_glFlush(unsigned long v)
 	glPolygonMode	(GL_FRONT, GL_FILL);
 
 	glDepthMask		(GL_TRUE);
-	glClear			(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+	glClear			(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 }
 
 void NDS_glPolygonAttrib (unsigned long val)
 {
-	u32 polygonID = (val>>24)&63;
+	//u32 polygonID = (val>>24)&63;
 
 	// Light enable/disable
 	lightMask = (val&0xF);
@@ -1149,8 +1190,8 @@ void NDS_glMaterial0 (unsigned long val)
 		glEnd();
 	}
 
-	glMaterialiv (GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
-	glMaterialiv (GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
+	glMaterialiv (GL_FRONT_AND_BACK, GL_AMBIENT, (GLint*)ambient);
+	glMaterialiv (GL_FRONT_AND_BACK, GL_DIFFUSE, (GLint*)diffuse);
 
 	if (beginCalled)
 	{
@@ -1174,8 +1215,8 @@ void NDS_glMaterial1 (unsigned long val)
 		glEnd();
 	}
 
-	glMaterialiv (GL_FRONT_AND_BACK, GL_SPECULAR, specular);
-	glMaterialiv (GL_FRONT_AND_BACK, GL_EMISSION, emission);
+	glMaterialiv (GL_FRONT_AND_BACK, GL_SPECULAR, (GLint*)specular);
+	glMaterialiv (GL_FRONT_AND_BACK, GL_EMISSION, (GLint*)emission);
 
 	if (beginCalled)
 	{
@@ -1204,16 +1245,16 @@ void NDS_glTexCoord(unsigned long val)
 
 	if (texCoordinateTransform == 1)
 	{
-		int s2 =(int)(	s*			mtxCurrent[3][0] + t*			mtxCurrent[3][4] +
-						(1.f/16.f)* mtxCurrent[3][8] + (1.f/16.f)*	mtxCurrent[3][12]);
-		int t2 =(int)(	s*			mtxCurrent[3][1] + t*			mtxCurrent[3][5] +
-						(1.f/16.f)* mtxCurrent[3][9] + (1.f/16.f)*	mtxCurrent[3][13]);
+		float s2 =(	s*			mtxCurrent[3][0] + t*			mtxCurrent[3][4] +
+					(1.f/16.f)* mtxCurrent[3][8] + (1.f/16.f)*	mtxCurrent[3][12]);
+		float t2 =(	s*			mtxCurrent[3][1] + t*			mtxCurrent[3][5] +
+					(1.f/16.f)* mtxCurrent[3][9] + (1.f/16.f)*	mtxCurrent[3][13]);
 
-		glTexCoord2i (s2, t2);
+		glTexCoord2f (s2, t2);
 	}
 	else
 	{
-		glTexCoord2i (s, t);
+		glTexCoord2f (s, t);
 	}
 }
 
@@ -1243,22 +1284,28 @@ signed long NDS_glGetDirectionalMatrix (unsigned int index)
 
 void NDS_glLightDirection (unsigned long v)
 {
-	float lightDirection[4] = {0};
-	lightDirection[0] = -normalTable[v&1023];
-	lightDirection[1] = -normalTable[(v>>10)&1023];
-	lightDirection[2] = -normalTable[(v>>20)&1023];
+	int		index = v>>30;
+	float	direction[4];
 
 	if (beginCalled)
-	{
 		glEnd();
-	}
 
-	glLightfv		(GL_LIGHT0 + (v>>30), GL_POSITION, lightDirection);
+	// Convert format into floating point value
+	// and pass it to OpenGL
+	direction[0] = -normalTable[v&1023];
+	direction[1] = -normalTable[(v>>10)&1023];
+	direction[2] = -normalTable[(v>>20)&1023];
+	direction[3] = 0;
+
+//	glLightf (GL_LIGHT0 + index, GL_CONSTANT_ATTENUATION, 0);
+
+	glLightfv(GL_LIGHT0 + index, GL_POSITION, direction);
 
 	if (beginCalled)
-	{
 		glBegin (vtxFormat);
-	}
+
+	// Keep information for GetLightDirection function
+	g_lightInfo[index].direction = v;
 }
 
 void NDS_glLightColor (unsigned long v)
@@ -1267,29 +1314,32 @@ void NDS_glLightColor (unsigned long v)
 							((v>> 5)&0x1F)<<26,
 							((v>>10)&0x1F)<<26,
 							0x7fffffff};
+	int index = v>>30;
 
 	if (beginCalled)
-	{
 		glEnd();
-	}
 
-	glLightiv (GL_LIGHT0 + (v>>30), GL_AMBIENT, lightColor);
-	glLightiv (GL_LIGHT0 + (v>>30), GL_DIFFUSE, lightColor);
-	glLightiv (GL_LIGHT0 + (v>>30), GL_SPECULAR, lightColor);
+	glLightiv(GL_LIGHT0 + index, GL_AMBIENT, (GLint*)lightColor);
+	glLightiv(GL_LIGHT0 + index, GL_DIFFUSE, (GLint*)lightColor);
+	glLightiv(GL_LIGHT0 + index, GL_SPECULAR, (GLint*)lightColor);
 
 	if (beginCalled)
-	{
 		glBegin (vtxFormat);
-	}
+
+	// Keep color information for NDS_glGetColor
+	g_lightInfo[index].color = v;
 }
 
 void NDS_glAlphaFunc(unsigned long v)
 {
-	glAlphaFunc (GL_GREATER, (v&31)/31.f);
+	alphaTestRef = (v&31)/31.f;
 }
 
 void NDS_glControl(unsigned long v)
 {
+	if(beginCalled)
+		glEnd();
+
 	if(v&1)
 	{
 		glEnable (GL_TEXTURE_2D);
@@ -1301,50 +1351,15 @@ void NDS_glControl(unsigned long v)
 
 	if(v&(1<<2))
 	{
-		glEnable(GL_ALPHA_TEST);
+		glAlphaFunc (GL_GREATER, 0.01f);
 	}
 	else
 	{
-		glDisable(GL_ALPHA_TEST);
+		glAlphaFunc (GL_GREATER, alphaTestRef);
 	}
 
-/*
-	// Seems to broke hell a lot of stuff
-	if(v&(1<<3))// && !disableBlending)
-	{
-		/*
-		if (alphaDepthWrite)
-		{
-			glDepthMask (GL_TRUE);
-		}
-		else
-		{
-			glDepthMask (GL_FALSE);
-			//glDisable	(GL_DEPTH_TEST);
-		}
-		*-/
-
-		glBlendFunc	(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-		//glBlendFunc	(GL_ONE, GL_ONE);
-		//glBlendFunc	(GL_SRC_ALPHA, GL_ONE);
-		glEnable		(GL_BLEND);
-	}
-	else
-	{
-		glDepthMask (GL_TRUE);
-		glDisable(GL_BLEND);
-	}
-*/
-	if(v&(1<<7))
-	{
-		int fog = 1;
-	}
-
-
-	if(v&(1<<4))
-	{
-		glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
-	}
+	if(beginCalled)
+		glBegin(vtxFormat);
 }
 
 void NDS_glNormal(unsigned long v)
@@ -1355,12 +1370,12 @@ void NDS_glNormal(unsigned long v)
 
 	if (texCoordinateTransform == 2)
 	{
-		int s2 =(int)(	(normal[0] *mtxCurrent[3][0] + normal[1] *mtxCurrent[3][4] +
-						 normal[2] *mtxCurrent[3][8]) + s);
-		int t2 =(int)(	(normal[0] *mtxCurrent[3][1] + normal[1] *mtxCurrent[3][5] +
-						 normal[2] *mtxCurrent[3][9]) + t);
+		float s2 =(	(normal[0] *mtxCurrent[3][0] + normal[1] *mtxCurrent[3][4] +
+					 normal[2] *mtxCurrent[3][8]) + s);
+		float t2 =(	(normal[0] *mtxCurrent[3][1] + normal[1] *mtxCurrent[3][5] +
+					 normal[2] *mtxCurrent[3][9]) + t);
 
-		glTexCoord2i (s2, t2);
+		glTexCoord2f (s2, t2);
 	}
 
 	MatrixMultVec3x3 (mtxCurrent[2], normal);
@@ -1370,7 +1385,7 @@ void NDS_glNormal(unsigned long v)
 
 void NDS_glCallList(unsigned long v)
 {
-	static unsigned long oldval = 0, shit = 0;
+	//static unsigned long oldval = 0, shit = 0;
 
 	if(!clInd)
 	{
@@ -1386,12 +1401,14 @@ void NDS_glCallList(unsigned long v)
 		{
 			case 0x0:
 			{
-				--clInd;
+				if (clInd > 0)
+				{
+					--clInd;
+					clCmd >>= 8;
+					continue;
+				}
 
-				if (!clInd)
-					break;
-
-				continue;
+				break;
 			}
 
 			case 0x11 :
@@ -1781,12 +1798,38 @@ void NDS_glCallList(unsigned long v)
 	}
 	if((clCmd&0xFF)==0x41)
 	{
-		glEnd();
+		if(beginCalled)
+		{
+			glEnd();
+			beginCalled = 0;
+		}
 		--clInd;
 		clCmd>>=8;
 	}
 }
 
+void NDS_glGetMatrix(unsigned int mode, unsigned int index, float* dest)
+{
+	//int n;
+
+	if(index == -1)
+	{
+		MatrixCopy(dest, mtxCurrent[mode]);
+		return;
+	}
+
+	MatrixCopy(dest, MatrixStackGetPos(&mtxStack[mode], index));
+}
+
+void NDS_glGetLightDirection(unsigned int index, unsigned int* dest)
+{
+	*dest = g_lightInfo[index].direction;
+}
+
+void NDS_glGetLightColor(unsigned int index, unsigned int* dest)
+{
+	*dest = g_lightInfo[index].color;
+}
 
 GPU3DInterface gpu3Dgl = {	NDS_glInit,
 							NDS_glViewPort,
@@ -1834,5 +1877,10 @@ GPU3DInterface gpu3Dgl = {	NDS_glInit,
 
 							NDS_glGetClipMatrix,
 							NDS_glGetDirectionalMatrix,
-							NDS_glGetLine};
+							NDS_glGetLine,
+
+							NDS_glGetMatrix,
+							NDS_glGetLightDirection,
+							NDS_glGetLightColor
+};
 
