@@ -21,8 +21,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-//#define RENDER3D
-
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <shellapi.h>
@@ -46,20 +44,28 @@
 #include "tileView.h"
 #include "oamView.h"
 #include "mapview.h"
+#include "matrixview.h"
+#include "lightview.h"
 #include "ConfigKeys.h"
+#include "FirmConfig.h"
+#include "AboutBox.h"
+#include "OGLRender.h"
+#include "../render3D.h"
+#include "../gdbstub.h"
+#include "colorctrl.h"
 
 #include "snddx.h"
 
-#ifdef RENDER3D
-     #include "OGLRender.h"
-#endif
+/* The compact flash disk image file */
+static const char *bad_glob_cflash_disk_image_file;
+static char cflash_filename_buffer[512];
 
 /*  Declare Windows procedure  */
 LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
 
 /*  Make the class name into a global variable  */
 char SavName[MAX_PATH] = "";
-char SavName2[MAX_PATH] = "";
+char ImportSavName[MAX_PATH] = "";
 char szClassName[ ] = "DeSmuME";
 int romnum = 0;
 
@@ -87,7 +93,11 @@ float heightTradeOff;
 HMENU menu;
 HANDLE runthread=INVALID_HANDLE_VALUE;
 
-const DWORD tabkey[48]={0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,VK_SPACE,VK_UP,VK_DOWN,VK_LEFT,VK_RIGHT,VK_TAB,VK_SHIFT,VK_DELETE,VK_INSERT,VK_HOME,VK_END,0x0d};
+const DWORD DI_tabkey[48] = {DIK_0,DIK_1,DIK_2,DIK_3,DIK_4,DIK_5,DIK_6,DIK_7,DIK_8,DIK_9,DIK_A,DIK_B,DIK_C,
+							DIK_D,DIK_E,DIK_F,DIK_G,DIK_H,DIK_I,DIK_J,DIK_K,DIK_L,DIK_M,DIK_N,DIK_O,DIK_P,
+							DIK_Q,DIK_R,DIK_S,DIK_T,DIK_U,DIK_V,DIK_W,DIK_X,DIK_Y,DIK_Z,DIK_SPACE,DIK_UP,
+							DIK_DOWN,DIK_LEFT,DIK_RIGHT,DIK_TAB,DIK_LSHIFT,DIK_DELETE,DIK_INSERT,DIK_HOME,
+							DIK_END,DIK_RETURN};
 DWORD ds_up,ds_down,ds_left,ds_right,ds_a,ds_b,ds_x,ds_y,ds_l,ds_r,ds_select,ds_start,ds_debug;
 static char IniName[MAX_PATH];
 int sndcoretype=SNDCORE_DIRECTX;
@@ -101,14 +111,95 @@ SoundInterface_struct *SNDCoreList[] = {
 NULL
 };
 
+GPU3DInterface *core3DList[] = {
+&gpu3DNull,
+&gpu3Dgl,
+};
+
 int autoframeskipenab=1;
 int frameskiprate=0;
 static int backupmemorytype=MC_TYPE_AUTODETECT;
 static u32 backupmemorysize=1;
 
+/* the firmware settings */
+struct NDS_fw_config_data win_fw_config;
+
+
 LRESULT CALLBACK SoundSettingsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
                                       LPARAM lParam);
-                                      
+
+struct configured_features {
+  u16 arm9_gdb_port;
+  u16 arm7_gdb_port;
+
+  const char *cflash_disk_image_file;
+};
+
+static void
+init_configured_features( struct configured_features *config) {
+  config->arm9_gdb_port = 0;
+  config->arm7_gdb_port = 0;
+
+  config->cflash_disk_image_file = NULL;
+}
+
+
+static int
+fill_configured_features( struct configured_features *config, LPSTR lpszArgument) {
+	int good_args = 0;
+	LPTSTR cmd_line;
+	LPWSTR *argv;
+	int argc;
+
+	argv = CommandLineToArgvW( GetCommandLineW(), &argc);
+
+	if ( argv != NULL) {
+		int i;
+		good_args = 1;
+		for ( i = 1; i < argc && good_args; i++) {
+			if ( wcsncmp( argv[i], L"--arm9gdb=", 10) == 0) {
+				wchar_t *end_char;
+				unsigned long port_num = wcstoul( &argv[i][10], &end_char, 10);
+
+				if ( port_num > 0 && port_num < 65536) {
+					config->arm9_gdb_port = port_num;
+				}
+				else {
+			        MessageBox(NULL,"ARM9 GDB stub port must be in the range 1 to 65535","Error",MB_OK);
+					good_args = 0;
+				}
+			}
+			else if ( wcsncmp( argv[i], L"--arm7gdb=", 10) == 0) {
+				wchar_t *end_char;
+				unsigned long port_num = wcstoul( &argv[i][10], &end_char, 10);
+
+				if ( port_num > 0 && port_num < 65536) {
+					config->arm7_gdb_port = port_num;
+				}
+				else {
+                                  MessageBox(NULL,"ARM9 GDB stub port must be in the range 1 to 65535","Error",MB_OK);
+					good_args = 0;
+				}
+			}
+			else if ( wcsncmp( argv[i], L"--cflash=", 9) == 0) {
+				if ( config->cflash_disk_image_file == NULL) {
+					size_t convert_count = wcstombs( &cflash_filename_buffer[0], &argv[i][9], 512);
+					if ( convert_count > 0) {
+						config->cflash_disk_image_file = cflash_filename_buffer;
+					}
+				}
+				else {
+			        MessageBox(NULL,"CFlash disk image file already set","Error",MB_OK);
+					good_args = 0;
+				}
+			}
+		}
+		LocalFree( argv);
+	}
+
+	return good_args;
+}
+
 // Rotation definitions
 u8    GPU_screenrotated[4*256*192];
 short GPU_rotation      = 0;
@@ -234,6 +325,49 @@ void translateXY(s32 *x, s32*y)
      
  // END Rotation definitions
 
+void Input_Post()
+{
+	char txt[255];
+	BOOL bPressed;
+	HRESULT hr;
+	WORD keys[13][3]={{ds_a,0xFFFE,0x01},{ds_b,0xFFFD,0x02},{ds_select,0xFFFB,0x04},{ds_start,0xFFF7,0x08},
+					{ds_right,0xFFEF,0x10},{ds_left,0xFFDF,0x20},{ds_up,0xFFBF,0x40},{ds_down,0xFF7F,0x80},
+					{ds_r,0xFEFF,0x100},{ds_l,0xFDFF,0x200},
+					{ds_x,0xFFFE,0x01},{ds_y,0xFFFD,0x02},{ds_debug,0xFFFB,0x04}};
+	int i;
+	
+	for (i=0; i<10; i++)
+	{
+		bPressed=FALSE;
+		if (keys[i][0]<48)
+			if (g_cDIBuf[DI_tabkey[keys[i][0]]]&0x80) bPressed=TRUE;
+		if (keys[i][0]>=48)
+			if (g_cDIBuf[208+keys[i][0]]&0x80) bPressed=TRUE;
+		if (bPressed)
+		{
+				((u16 *)ARM9Mem.ARM9_REG)[0x130>>1]&=keys[i][1];
+				((u16 *)MMU.ARM7_REG)[0x130>>1]&=keys[i][1];
+		}
+		else
+		{
+				((u16 *)ARM9Mem.ARM9_REG)[0x130>>1]|=keys[i][2];
+				((u16 *)MMU.ARM7_REG)[0x130>>1]|=keys[i][2];
+		}
+	}
+	for (i=10; i<13; i++)
+	{
+		bPressed=FALSE;
+		if (keys[i][0]<48)
+			if (g_cDIBuf[DI_tabkey[keys[i][0]]]&0x80) bPressed=TRUE;
+		if (keys[i][0]>=48)
+			if (g_cDIBuf[208+keys[i][0]]&0x80) bPressed=TRUE;
+		if (bPressed)
+				((u16 *)MMU.ARM7_REG)[0x136>>1]&=keys[i][1];
+		else
+				((u16 *)MMU.ARM7_REG)[0x136>>1]|=keys[i][2];
+	}
+}
+
 DWORD WINAPI run( LPVOID lpParameter)
 {
      char txt[80];
@@ -278,9 +412,14 @@ DWORD WINAPI run( LPVOID lpParameter)
      rotationbmi.bV4Width = 256;
      rotationbmi.bV4Height = -192;
 
-     #ifdef RENDER3D
-     OGLRender::init(&hdc);
-     #endif
+     NDS_3D_SetDriver (GPU3D_OPENGL);
+
+	if (!gpu3D->NDS_3D_Init ())
+	{
+		MessageBox(hwnd,"Unable to initialize openGL","Error",MB_OK);
+		return -1;
+    }
+
      QueryPerformanceFrequency((LARGE_INTEGER *)&freq);
      QueryPerformanceCounter((LARGE_INTEGER *)&lastticks);
      OneFrameTime = freq / 60;
@@ -291,6 +430,8 @@ DWORD WINAPI run( LPVOID lpParameter)
           {
                cycles = NDS_exec((560190<<1)-cycles,FALSE);
                SPU_Emulate();
+			   Input_Process();
+			   Input_Post();
 
                if (!skipnextframe)
                {
@@ -374,7 +515,7 @@ DWORD WINAPI run( LPVOID lpParameter)
                else
                {
                   if (framestoskip < 1)
-                     framestoskip = frameskiprate + 1;
+                     framestoskip += frameskiprate;
                }
 
                CWindow_RefreshALL();
@@ -414,33 +555,141 @@ void StateLoadSlot(int num)
 	NDS_UnPause();
 }
 
-BOOL LoadROM(char * filename)
+BOOL LoadROM(char * filename, const char *cflash_disk_image)
 {
     NDS_Pause();
 
-    if (NDS_LoadROM(filename, backupmemorytype, backupmemorysize) > 0)
+    if (NDS_LoadROM(filename, backupmemorytype, backupmemorysize, cflash_disk_image) > 0)
        return TRUE;
 
     return FALSE;
 }
 
+/*
+ * The thread handling functions needed by the GDB stub code.
+ */
+void *
+createThread_gdb( void (*thread_function)( void *data),
+				 void *thread_data) {
+	void *new_thread = CreateThread( NULL, 0,
+		(LPTHREAD_START_ROUTINE)thread_function, thread_data,
+		0, NULL);
+
+	return new_thread;
+}
+
+void
+joinThread_gdb( void *thread_handle) {
+}
+
+
+void SetLanguage(int langid)
+{
+   switch(langid)
+   {
+      case 1:
+         // French
+         SetThreadLocale(MAKELCID(MAKELANGID(LANG_FRENCH, SUBLANG_FRENCH),
+                         SORT_DEFAULT));          
+         break;
+      case 2:
+         // Danish
+         SetThreadLocale(MAKELCID(MAKELANGID(LANG_DANISH, SUBLANG_DEFAULT),
+                         SORT_DEFAULT));          
+         break;
+      case 0:
+         // English
+         SetThreadLocale(MAKELCID(MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),
+                         SORT_DEFAULT));
+         break;
+      default: break;
+         break;
+   }
+}
+
+void SaveLanguage(int langid)
+{
+   char text[80];
+
+   sprintf(text, "%d", langid);
+   WritePrivateProfileString("General", "Language", text, IniName);
+}
+
+void CheckLanguage(UINT id)
+{
+   int i;
+   for (i = IDC_LANGENGLISH; i < IDC_LANGFRENCH+1; i++)
+      CheckMenuItem(menu, i, MF_BYCOMMAND | MF_UNCHECKED);
+
+   CheckMenuItem(menu, id, MF_BYCOMMAND | MF_CHECKED);
+}
+
+void ChangeLanguage(int id)
+{
+   HMENU newmenu;
+
+   SetLanguage(id);
+   newmenu = LoadMenu(hAppInst, "MENU_PRINCIPAL");
+   SetMenu(hwnd, newmenu);
+   DestroyMenu(menu);
+   menu = newmenu;   
+}
+
+void InitCustomControls()
+{
+	ColorCtrl_Register();
+}
+
 int WINAPI WinMain (HINSTANCE hThisInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR lpszArgument,
                     int nFunsterStil)
 
 {
-    MSG messages;            /* Here messages to the application are saved */
+#ifdef GDB_STUB
+	gdbstub_handle_t arm9_gdb_stub;
+	gdbstub_handle_t arm7_gdb_stub;
+	struct armcpu_memory_iface *arm9_memio = &arm9_base_memory_iface;
+	struct armcpu_memory_iface *arm7_memio = &arm7_base_memory_iface;
+	struct armcpu_ctrl_iface *arm9_ctrl_iface;
+	struct armcpu_ctrl_iface *arm7_ctrl_iface;
+#endif
+
+	struct configured_features my_config;
+
+
+	MSG messages;            /* Here messages to the application are saved */
     char text[80];
     cwindow_struct MainWindow;
     HACCEL hAccel;
     hAppInst=hThisInstance;
 
+	InitCustomControls();
+
+	/* default the firmware settings, they may get changed later */
+	NDS_FillDefaultFirmwareConfigData( &win_fw_config);
+
     InitializeCriticalSection(&section);
+
+    GetINIPath(IniName, MAX_PATH);
+    GetPrivateProfileString("General", "Language", "-1", text, 80, IniName);
+    SetLanguage(atoi(text));
+
     sprintf(text, "DeSmuME v%s", VERSION);
 
+	init_configured_features( &my_config);
+	if ( !fill_configured_features( &my_config, lpszArgument)) {
+        MessageBox(NULL,"Unable to parse command line arguments","Error",MB_OK);
+        return 0;
+	}
+	bad_glob_cflash_disk_image_file = my_config.cflash_disk_image_file;
+
     hAccel = LoadAccelerators(hAppInst, MAKEINTRESOURCE(IDR_MAIN_ACCEL));
 
+#ifdef DEBUG
+    LogStart();
+#endif
+
     if (CWindow_Init(&MainWindow, hThisInstance, szClassName, text,
                      WS_CAPTION| WS_SYSMENU | WS_SIZEBOX | WS_MINIMIZEBOX | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      256, 384, WindowProcedure) != 0)
@@ -463,13 +712,48 @@ int WINAPI WinMain (HINSTANCE hThisInstance,
     InitTileViewBox();
     InitOAMViewBox();
     
-#ifdef DEBUG
-    LogStart();
+#ifdef GDB_STUB
+	if ( my_config.arm9_gdb_port != 0) {
+		arm9_gdb_stub = createStub_gdb( my_config.arm9_gdb_port,
+			&arm9_memio, &arm9_direct_memory_iface);
+
+		if ( arm9_gdb_stub == NULL) {
+	       MessageBox(hwnd,"Failed to create ARM9 gdbstub","Error",MB_OK);
+		   return -1;
+		}
+	}
+	if ( my_config.arm7_gdb_port != 0) {
+		arm7_gdb_stub = createStub_gdb( my_config.arm7_gdb_port,
+                                    &arm7_memio,
+                                    &arm7_base_memory_iface);
+
+		if ( arm7_gdb_stub == NULL) {
+	       MessageBox(hwnd,"Failed to create ARM7 gdbstub","Error",MB_OK);
+		   return -1;
+		}
+	}
+
+    NDS_Init( arm9_memio, &arm9_ctrl_iface,
+		arm7_memio, &arm7_ctrl_iface);
+#else
+	NDS_Init ();
 #endif
 
-    GetINIPath(IniName);
 
-    NDS_Init();
+  /*
+   * Activate the GDB stubs
+   * This has to come after the NDS_Init where the cpus are set up.
+   */
+#ifdef GDB_STUB
+	if ( my_config.arm9_gdb_port != 0) {
+		activateStub_gdb( arm9_gdb_stub, arm9_ctrl_iface);
+	}
+	if ( my_config.arm7_gdb_port != 0) {
+		activateStub_gdb( arm7_gdb_stub, arm7_ctrl_iface);
+	}
+#endif
+    GetPrivateProfileString("General", "Language", "0", text, 80, IniName);
+    CheckLanguage(IDC_LANGENGLISH+atoi(text));
 
     GetPrivateProfileString("Video", "FrameSkip", "AUTO", text, 80, IniName);
 
@@ -492,20 +776,60 @@ int WINAPI WinMain (HINSTANCE hThisInstance,
     if (SPU_ChangeSoundCore(sndcoretype, sndbuffersize) != 0)
     {
        MessageBox(hwnd,"Unable to initialize DirectSound","Error",MB_OK);
-       return messages.wParam;
+       return -1;
     }
 
-    sndvolume = GetPrivateProfileInt("Sound","Volume",100, IniName);
+	sndvolume = GetPrivateProfileInt("Sound","Volume",100, IniName);
     SPU_SetVolume(sndvolume);
-    
+
+	/* Read the firmware settings from the init file */
+    win_fw_config.fav_colour = GetPrivateProfileInt("Firmware","favColor", 10, IniName);
+    win_fw_config.birth_month = GetPrivateProfileInt("Firmware","bMonth", 7, IniName);
+    win_fw_config.birth_day = GetPrivateProfileInt("Firmware","bDay", 15, IniName);
+    win_fw_config.language = GetPrivateProfileInt("Firmware","Language", 1, IniName);
+
+	{
+		/*
+		 * Read in the nickname and message.
+		 * Convert the strings into Unicode UTF-16 characters.
+		 */
+		char temp_str[27];
+		int char_index;
+	    GetPrivateProfileString("Firmware","nickName", "yopyop", temp_str, 11, IniName);
+		win_fw_config.nickname_len = strlen( temp_str);
+
+		if ( win_fw_config.nickname_len == 0) {
+			strcpy( temp_str, "yopyop");
+			win_fw_config.nickname_len = strlen( temp_str);
+		}
+
+		for ( char_index = 0; char_index < win_fw_config.nickname_len; char_index++) {
+			win_fw_config.nickname[char_index] = temp_str[char_index];
+		}
+
+		GetPrivateProfileString("Firmware","Message", "DeSmuME makes you happy!", temp_str, 27, IniName);
+	    win_fw_config.message_len = strlen( temp_str);
+		for ( char_index = 0; char_index < win_fw_config.message_len; char_index++) {
+			win_fw_config.message[char_index] = temp_str[char_index];
+		}
+	}
+
+	/* Create the dummy firmware */
+	NDS_CreateDummyFirmware( &win_fw_config);
+
     runthread = CreateThread(NULL, 0, run, NULL, 0, &threadID);
-    
-    if(LoadROM(lpszArgument))
+
+    // Make sure any quotes from lpszArgument are removed
+    if (lpszArgument[0] == '\"')
+       sscanf(lpszArgument, "\"%[^\"]\"", lpszArgument);
+
+    if(LoadROM(lpszArgument, bad_glob_cflash_disk_image_file))
     {
        EnableMenuItem(menu, IDM_EXEC, MF_GRAYED);
        EnableMenuItem(menu, IDM_PAUSE, MF_ENABLED);
        EnableMenuItem(menu, IDM_RESET, MF_ENABLED);
        EnableMenuItem(menu, IDM_GAME_INFO, MF_ENABLED);
+       EnableMenuItem(menu, IDM_IMPORTBACKUPMEMORY, MF_ENABLED);
        romloaded = TRUE;
        NDS_UnPause();
     }
@@ -515,6 +839,7 @@ int WINAPI WinMain (HINSTANCE hThisInstance,
        EnableMenuItem(menu, IDM_PAUSE, MF_GRAYED);
        EnableMenuItem(menu, IDM_RESET, MF_GRAYED);
        EnableMenuItem(menu, IDM_GAME_INFO, MF_GRAYED);
+       EnableMenuItem(menu, IDM_IMPORTBACKUPMEMORY, MF_GRAYED);
     }
 
     CheckMenuItem(menu, IDC_SAVETYPE1, MF_BYCOMMAND | MF_CHECKED);
@@ -536,7 +861,13 @@ int WINAPI WinMain (HINSTANCE hThisInstance,
           DispatchMessage(&messages);
        }  
     }
-    
+	{
+	HRESULT hr=Input_DeInit();
+#ifdef DEBUG 
+	if(FAILED(hr)) LOG("DirectInput deinit failed (0x%08X)\n",hr);
+	else LOG("DirectInput deinit\n");
+#endif
+	}
 #ifdef DEBUG
     LogStop();
 #endif
@@ -551,7 +882,11 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
         case WM_CREATE:
 	     {
 	     	RECT clientSize, fullSize;
-             	ReadConfig();
+             	HRESULT hr=Input_Init(hwnd);
+#ifdef DEBUG
+				if(FAILED(hr)) LOG("DirectInput init failed (0x%08x)\n",hr);
+				else LOG("DirectInput init\n");
+#endif
 	     	GetClientRect(hwnd, &clientSize);
 			GetWindowRect(hwnd, &fullSize);
 	     	DefaultWidth = clientSize.right - clientSize.left;
@@ -561,6 +896,7 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
             return 0;
 	     }
         case WM_DESTROY:
+			{
              NDS_Pause();
              finished = TRUE;
 
@@ -578,6 +914,7 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
 
              PostQuitMessage (0);       // send a WM_QUIT to the message queue 
              return 0;
+			}
 	case WM_SIZE:
     	if (ForceRatio) {
 			RECT fullSize;
@@ -586,6 +923,7 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
 		}
 	     return 0;
         case WM_CLOSE:
+			{
              NDS_Pause();
              finished = TRUE;
 
@@ -602,146 +940,25 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
              NDS_DeInit();
              PostMessage(hwnd, WM_QUIT, 0, 0);
              return 0;
+			}
         case WM_DROPFILES:
              {
                   char filename[MAX_PATH] = "";
                   DragQueryFile((HDROP)wParam,0,filename,MAX_PATH);
                   DragFinish((HDROP)wParam);
-                  if(LoadROM(filename))
+                  if(LoadROM(filename, bad_glob_cflash_disk_image_file))
                   {
                      EnableMenuItem(menu, IDM_EXEC, MF_GRAYED);
                      EnableMenuItem(menu, IDM_PAUSE, MF_ENABLED);
                      EnableMenuItem(menu, IDM_RESET, MF_ENABLED);
                      EnableMenuItem(menu, IDM_GAME_INFO, MF_ENABLED);
+                     EnableMenuItem(menu, IDM_IMPORTBACKUPMEMORY, MF_ENABLED);
                      romloaded = TRUE;
                      NDS_UnPause();
                   }
              }
              return 0;
-        case WM_KEYDOWN:
-             //if(wParam=='1'){PostMessage(hwnd, WM_DESTROY, 0, 0);return 0;}
-             
-             if(wParam==tabkey[ds_a]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFFE;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFFE;
-             return 0; }
-             if(wParam==tabkey[ds_b]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFFD;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFFD;
-             return 0; }
-             if(wParam==tabkey[ds_select]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFFB;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFFB;
-             return 0; }
-             if(wParam==tabkey[ds_start]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFF7;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFF7;
-             return 0; }
-             if(wParam==tabkey[ds_right]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFEF;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFEF;
-             return 0; }
-             if(wParam==tabkey[ds_left]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFDF;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFDF;
-             return 0; }
-             if(wParam==tabkey[ds_up]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFFBF;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFFBF;
-             return 0; }
-             if(wParam==tabkey[ds_down]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFF7F;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFF7F;
-             return 0; }
-             if(wParam==tabkey[ds_r]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFEFF;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFEFF;
-             return 0; }
-             if(wParam==tabkey[ds_l]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] &= 0xFDFF;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] &= 0xFDFF;
-             return 0; }
-             if(wParam==tabkey[ds_x]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] &= 0xFFFE;
-             return 0; }
-             if(wParam==tabkey[ds_y]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] &= 0xFFFD;
-             return 0; }
-             if(wParam==tabkey[ds_debug]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] &= 0xFFFB;
-             return 0; }
-             return 0;
-                  /*case 0x1E :
-                       MMU.ARM7_REG[0x136] &= 0xFE;
-                       break;
-                  case 0x1F :
-                       MMU.ARM7_REG[0x136] &= 0xFD;
-                       break;
-                  case 0x21 :
-                       NDS_ARM9.wIRQ = TRUE;
-                       break; */
-        case WM_KEYUP:
-             if(wParam==tabkey[ds_a]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x1;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x1;
-             return 0; }
-             if(wParam==tabkey[ds_b]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x2;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x2;
-             return 0; }
-             if(wParam==tabkey[ds_select]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x4;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x4;
-             return 0; }
-             if(wParam==tabkey[ds_start]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x8;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x8;
-             return 0; }
-             if(wParam==tabkey[ds_right]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x10;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x10;
-             return 0; }
-             if(wParam==tabkey[ds_left]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x20;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x20;
-             return 0; }
-             if(wParam==tabkey[ds_up]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x40;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x40;
-             return 0; }
-             if(wParam==tabkey[ds_down]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x80;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x80;
-             return 0; }
-             if(wParam==tabkey[ds_r]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x100;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x100;
-             return 0; }
-             if(wParam==tabkey[ds_l]){
-             ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] |= 0x200;
-             ((u16 *)MMU.ARM7_REG)[0x130>>1] |= 0x200;
-             return 0; }
-             if(wParam==tabkey[ds_x]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] |= 0x1;
-             return 0; }
-             if(wParam==tabkey[ds_y]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] |= 0x2;
-             return 0; }
-             if(wParam==tabkey[ds_debug]){
-             ((u16 *)MMU.ARM7_REG)[0x136>>1] |= 0x4;
-             return 0; }
-             break;
-                       
-                  /*case 0x1E :
-                       MMU.ARM7_REG[0x136] |= 1;
-                       break;
-                  case 0x1F :
-                       MMU.ARM7_REG[0x136] |= 2;
-                       break;*/
-                  /*case 0x21 :
-                       MMU.REG_IME[0] = 1;*/
         case WM_MOUSEMOVE:
-             {
                   if (wParam & MK_LBUTTON)
                   {
 					   RECT r ;
@@ -772,49 +989,48 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                        NDS_setTouchPos(x, y);
                        return 0;
                   }
-             }
-             NDS_releasTouch();
+				NDS_releasTouch();
              return 0;
         case WM_LBUTTONDOWN:
-             if(HIWORD(lParam)>=192)
-             {
-					   RECT r ;
-                s32 x = (s32)((s16)LOWORD(lParam));
-                s32 y = (s32)((s16)HIWORD(lParam));
-						GetClientRect(hwnd,&r) ;
-						/* translate from scaling (screen reoltution to 256x384 or 512x192) */
-					   switch (GPU_rotation)
-						{
-							case 0:
-							case 180:
-								x = (x*256) / (r.right - r.left) ;
-								y = (y*384) / (r.bottom - r.top) ;
-								break ;
-							case 90:
-							case 270:
-								x = (x*512) / (r.right - r.left) ;
-								y = (y*192) / (r.bottom - r.top) ;
-								break ;
-						}
-						/* translate for rotation */
-                if (GPU_rotation != 0)
-                   translateXY(&x,&y);
-                else
-                  y-=192;
-                if(y>=0)
-                {
-                     SetCapture(hwnd);
-                     if(x<0) x = 0; else if(x>255) x = 255;
-                     if(y<0) y = 0; else if(y>192) y = 192;
-                     NDS_setTouchPos(x, y);
-                     click = TRUE;
-                }
-             }
+				if(HIWORD(lParam)>=192)
+				{
+						   RECT r ;
+					s32 x = (s32)((s16)LOWORD(lParam));
+					s32 y = (s32)((s16)HIWORD(lParam));
+							GetClientRect(hwnd,&r) ;
+							/* translate from scaling (screen reoltution to 256x384 or 512x192) */
+						   switch (GPU_rotation)
+							{
+								case 0:
+								case 180:
+									x = (x*256) / (r.right - r.left) ;
+									y = (y*384) / (r.bottom - r.top) ;
+									break ;
+								case 90:
+								case 270:
+									x = (x*512) / (r.right - r.left) ;
+									y = (y*192) / (r.bottom - r.top) ;
+									break ;
+							}
+							/* translate for rotation */
+					if (GPU_rotation != 0)
+					   translateXY(&x,&y);
+					else
+					  y-=192;
+					if(y>=0)
+					{
+						 SetCapture(hwnd);
+						 if(x<0) x = 0; else if(x>255) x = 255;
+						 if(y<0) y = 0; else if(y>192) y = 192;
+						 NDS_setTouchPos(x, y);
+						 click = TRUE;
+					}
+				 }
              return 0;
         case WM_LBUTTONUP:
-             if(click)
-                  ReleaseCapture();
-             NDS_releasTouch();
+				if(click)
+					  ReleaseCapture();
+				NDS_releasTouch();
              return 0;
         case WM_COMMAND:
              switch(LOWORD(wParam))
@@ -864,12 +1080,13 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                             
                             LOG("%s\r\n", filename);
 
-                            if(LoadROM(filename))
+                            if(LoadROM(filename, bad_glob_cflash_disk_image_file))
                             {
                                EnableMenuItem(menu, IDM_EXEC, MF_GRAYED);
                                EnableMenuItem(menu, IDM_PAUSE, MF_ENABLED);
                                EnableMenuItem(menu, IDM_RESET, MF_ENABLED);
                                EnableMenuItem(menu, IDM_GAME_INFO, MF_ENABLED);
+                               EnableMenuItem(menu, IDM_IMPORTBACKUPMEMORY, MF_ENABLED);
                                romloaded = TRUE;
                                NDS_UnPause();
                             }
@@ -1002,6 +1219,30 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                   case IDM_STATE_LOAD_F10:
                      StateLoadSlot(10);
                      return 0;
+                  case IDM_IMPORTBACKUPMEMORY:
+                  {
+                     OPENFILENAME ofn;
+                     NDS_Pause();
+                     ZeroMemory(&ofn, sizeof(ofn));
+                     ofn.lStructSize = sizeof(ofn);
+                     ofn.hwndOwner = hwnd;
+                     ofn.lpstrFilter = "All supported types\0*.duc;*.sav\0Action Replay DS Save (*.duc)\0*.duc\0DS-Xtreme Save (*.sav)\0*.sav\0\0";
+                     ofn.nFilterIndex = 1;
+                     ofn.lpstrFile =  ImportSavName;
+                     ofn.nMaxFile = MAX_PATH;
+                     ofn.lpstrDefExt = "duc";
+                            
+                     if(!GetOpenFileName(&ofn))
+                     {
+                        NDS_UnPause();
+                        return 0;
+                     }
+
+                     if (!NDS_ImportSave(ImportSavName))
+                        MessageBox(hwnd,"Save was not successfully imported","Error",MB_OK);
+                     NDS_UnPause();
+                     return 0;
+                  }
                   case IDM_SOUNDSETTINGS:
                   {
                       DialogBox(GetModuleHandle(NULL), "SoundSettingsDlg", hwnd, (DLGPROC)SoundSettingsDlgProc);                    
@@ -1080,7 +1321,7 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                   return 0;
                   case IDM_OAM:
                        {
-                            oamview_struct *OamView;
+						   oamview_struct *OamView;
 
                             if ((OamView = OamView_Init(hAppInst, HWND_DESKTOP)) != NULL)
                             {
@@ -1089,7 +1330,30 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                             }
                        }
                   return 0;
-                  case IDM_MBG0 : 
+
+				  case IDM_MATRIX_VIEWER:
+                       {
+                            matrixview_struct *MatrixView;
+
+                            if ((MatrixView = MatrixView_Init(hAppInst, HWND_DESKTOP)) != NULL)
+                            {
+                               CWindow_Show(MatrixView);
+                            }
+                       }
+                  return 0;
+
+				  case IDM_LIGHT_VIEWER:
+                       {
+                            lightview_struct *LightView;
+
+                            if ((LightView = LightView_Init(hAppInst, HWND_DESKTOP)) != NULL)
+                            {
+                               CWindow_Show(LightView);
+                            }
+                       }
+                  return 0;
+
+				  case IDM_MBG0 : 
                        if(MainScreen.gpu->dispBG[0])
                        {
                             GPU_remove(MainScreen.gpu, 0);
@@ -1241,6 +1505,16 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
 
                        }
                   return 0;
+                  case IDM_FIRMSETTINGS:
+                       {
+                            cwindow_struct FirmConfig;
+
+                            if (CWindow_Init2(&FirmConfig, hAppInst, HWND_DESKTOP,
+									"Configure Controls", IDD_FIRMSETTINGS, FirmConfig_Proc) == 0)
+                               CWindow_Show(&FirmConfig);
+
+                       }
+                  return 0;
                   case IDC_FRAMESKIPAUTO:
                   case IDC_FRAMESKIP0:
                   case IDC_FRAMESKIP1:
@@ -1281,12 +1555,37 @@ LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM
                        CheckMenuItem(menu, LOWORD(wParam), MF_BYCOMMAND | MF_CHECKED);
                   }
                   return 0;
+                  case IDC_LANGENGLISH:
+                     SaveLanguage(0);
+                     ChangeLanguage(0);
+                     CheckLanguage(LOWORD(wParam));
+                  return 0;
+                  case IDC_LANGFRENCH:
+                     SaveLanguage(1);
+                     ChangeLanguage(1);
+                     CheckLanguage(LOWORD(wParam));
+                  return 0;
+                  case IDC_LANGDANISH:
+                     SaveLanguage(1);
+                     ChangeLanguage(2);
+                     CheckLanguage(LOWORD(wParam));
+                  return 0;
                   case IDM_WEBSITE:
                        ShellExecute(NULL, "open", "http://desmume.sourceforge.net", NULL, NULL, SW_SHOWNORMAL);
                   return 0;
                   case IDM_FORUM:
-                       ShellExecute(NULL, "open", "http://sourceforge.net/forum/?group_id=164579", NULL, NULL, SW_SHOWNORMAL);
+                       ShellExecute(NULL, "open", "http://forums.desmume.org/index.php", NULL, NULL, SW_SHOWNORMAL);
                   return 0;
+
+				  case IDM_ABOUT:
+				  {
+					  cwindow_struct aboutBox;
+
+					  if (CWindow_Init2(&aboutBox, hAppInst, HWND_DESKTOP, "About desmume...", IDD_ABOUT_BOX, AboutBox_Proc) == 0)
+							CWindow_Show(&aboutBox);
+
+					  break;
+				  }
                   case IDM_SUBMITBUGREPORT:
                        ShellExecute(NULL, "open", "http://sourceforge.net/tracker/?func=add&group_id=164579&atid=832291", NULL, NULL, SW_SHOWNORMAL);
                   return 0;
@@ -1477,3 +1776,4 @@ LRESULT CALLBACK SoundSettingsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    return FALSE;
 }
 
+
