@@ -243,4 +243,120 @@ void refreshAll();
 #define IDC_EDIT1           1102
 #define IDC_EDIT2           1103
 
+#define IDD_ABOUT_BOX          1301
+#define IDM_ABOUT              40003
+#define IDC_AUTHORS_LIST	   1302
+
+// ============================================================================
+// Matrix Viewer ID's (1200 to 1299)
+// ============================================================================
+
+#define IDD_MATRIX_VIEWER							1200
+#define IDM_MATRIX_VIEWER							IDD_MATRIX_VIEWER
+
+// Position
+#define IDC_MATRIX_VIEWER_COORD_GROUP				1201
+#define IDC_MATRIX_VIEWER_COORD_COMBO				1202
+#define IDC_MATRIX_VIEWER_COORD_11_EDIT				1203
+#define IDC_MATRIX_VIEWER_COORD_12_EDIT				1204
+#define IDC_MATRIX_VIEWER_COORD_13_EDIT				1205
+#define IDC_MATRIX_VIEWER_COORD_14_EDIT				1206
+#define IDC_MATRIX_VIEWER_COORD_21_EDIT				1207
+#define IDC_MATRIX_VIEWER_COORD_22_EDIT				1208
+#define IDC_MATRIX_VIEWER_COORD_23_EDIT				1209
+#define IDC_MATRIX_VIEWER_COORD_24_EDIT				1210
+#define IDC_MATRIX_VIEWER_COORD_31_EDIT				1211
+#define IDC_MATRIX_VIEWER_COORD_32_EDIT				1212
+#define IDC_MATRIX_VIEWER_COORD_33_EDIT				1213
+#define IDC_MATRIX_VIEWER_COORD_34_EDIT				1214
+#define IDC_MATRIX_VIEWER_COORD_41_EDIT				1215
+#define IDC_MATRIX_VIEWER_COORD_42_EDIT				1216
+#define IDC_MATRIX_VIEWER_COORD_43_EDIT				1217
+#define IDC_MATRIX_VIEWER_COORD_44_EDIT				1218
+
+// Direction
+#define IDC_MATRIX_VIEWER_DIR_GROUP					1221
+#define IDC_MATRIX_VIEWER_DIR_COMBO					1222
+#define IDC_MATRIX_VIEWER_DIR_11_EDIT				1223
+#define IDC_MATRIX_VIEWER_DIR_12_EDIT				1224
+#define IDC_MATRIX_VIEWER_DIR_13_EDIT				1225
+#define IDC_MATRIX_VIEWER_DIR_14_EDIT				1226
+#define IDC_MATRIX_VIEWER_DIR_21_EDIT				1227
+#define IDC_MATRIX_VIEWER_DIR_22_EDIT				1228
+#define IDC_MATRIX_VIEWER_DIR_23_EDIT				1229
+#define IDC_MATRIX_VIEWER_DIR_24_EDIT				1230
+#define IDC_MATRIX_VIEWER_DIR_31_EDIT				1231
+#define IDC_MATRIX_VIEWER_DIR_32_EDIT				1232
+#define IDC_MATRIX_VIEWER_DIR_33_EDIT				1233
+#define IDC_MATRIX_VIEWER_DIR_34_EDIT				1234
+#define IDC_MATRIX_VIEWER_DIR_41_EDIT				1235
+#define IDC_MATRIX_VIEWER_DIR_42_EDIT				1236
+#define IDC_MATRIX_VIEWER_DIR_43_EDIT				1237
+#define IDC_MATRIX_VIEWER_DIR_44_EDIT				1238
+
+// Projection
+#define IDC_MATRIX_VIEWER_PROJ_GROUP				1241
+#define IDC_MATRIX_VIEWER_PROJ_11_EDIT				1243
+#define IDC_MATRIX_VIEWER_PROJ_12_EDIT				1244
+#define IDC_MATRIX_VIEWER_PROJ_13_EDIT				1245
+#define IDC_MATRIX_VIEWER_PROJ_14_EDIT				1246
+#define IDC_MATRIX_VIEWER_PROJ_21_EDIT				1247
+#define IDC_MATRIX_VIEWER_PROJ_22_EDIT				1248
+#define IDC_MATRIX_VIEWER_PROJ_23_EDIT				1249
+#define IDC_MATRIX_VIEWER_PROJ_24_EDIT				1250
+#define IDC_MATRIX_VIEWER_PROJ_31_EDIT				1251
+#define IDC_MATRIX_VIEWER_PROJ_32_EDIT				1252
+#define IDC_MATRIX_VIEWER_PROJ_33_EDIT				1253
+#define IDC_MATRIX_VIEWER_PROJ_34_EDIT				1254
+#define IDC_MATRIX_VIEWER_PROJ_41_EDIT				1255
+#define IDC_MATRIX_VIEWER_PROJ_42_EDIT				1256
+#define IDC_MATRIX_VIEWER_PROJ_43_EDIT				1257
+#define IDC_MATRIX_VIEWER_PROJ_44_EDIT				1258
+
+// Texture
+#define IDC_MATRIX_VIEWER_TEX_GROUP					1261
+#define IDC_MATRIX_VIEWER_TEX_11_EDIT				1263
+#define IDC_MATRIX_VIEWER_TEX_12_EDIT				1264
+#define IDC_MATRIX_VIEWER_TEX_13_EDIT				1265
+#define IDC_MATRIX_VIEWER_TEX_14_EDIT				1266
+#define IDC_MATRIX_VIEWER_TEX_21_EDIT				1267
+#define IDC_MATRIX_VIEWER_TEX_22_EDIT				1268
+#define IDC_MATRIX_VIEWER_TEX_23_EDIT				1269
+#define IDC_MATRIX_VIEWER_TEX_24_EDIT				1270
+#define IDC_MATRIX_VIEWER_TEX_31_EDIT				1271
+#define IDC_MATRIX_VIEWER_TEX_32_EDIT				1272
+#define IDC_MATRIX_VIEWER_TEX_33_EDIT				1273
+#define IDC_MATRIX_VIEWER_TEX_34_EDIT				1274
+#define IDC_MATRIX_VIEWER_TEX_41_EDIT				1275
+#define IDC_MATRIX_VIEWER_TEX_42_EDIT				1276
+#define IDC_MATRIX_VIEWER_TEX_43_EDIT				1277
+#define IDC_MATRIX_VIEWER_TEX_44_EDIT				1278
+
+// ============================================================================
+// Light Viewer ID's (1300 to 1399)
+// ============================================================================
+#define IDD_LIGHT_VIEWER							1300
+#define IDM_LIGHT_VIEWER							IDD_LIGHT_VIEWER
+
+#define IDC_LIGHT_VIWER_LIGHT0_GROUP				1301
+#define IDC_LIGHT_VIEWER_LIGHT0COLOR_COLORCTRL		1302
+#define IDC_LIGHT_VIEWER_LIGHT0COLOR_EDIT			1303
+#define IDC_LIGHT_VIEWER_LIGHT0VECTOR_EDIT			1304
+
+#define IDC_LIGHT_VIWER_LIGHT1_GROUP				1311
+#define IDC_LIGHT_VIEWER_LIGHT1COLOR_COLORCTRL		1312
+#define IDC_LIGHT_VIEWER_LIGHT1COLOR_EDIT			1313
+#define IDC_LIGHT_VIEWER_LIGHT1VECTOR_EDIT			1314
+
+#define IDC_LIGHT_VIWER_LIGHT2_GROUP				1321
+#define IDC_LIGHT_VIEWER_LIGHT2COLOR_COLORCTRL		1322
+#define IDC_LIGHT_VIEWER_LIGHT2COLOR_EDIT			1323
+#define IDC_LIGHT_VIEWER_LIGHT2VECTOR_EDIT			1324
+
+#define IDC_LIGHT_VIWER_LIGHT3_GROUP				1331
+#define IDC_LIGHT_VIEWER_LIGHT3COLOR_COLORCTRL		1332
+#define IDC_LIGHT_VIEWER_LIGHT3COLOR_EDIT			1333
+#define IDC_LIGHT_VIEWER_LIGHT3VECTOR_EDIT			1334
+
+
 #endif
